@@ -376,6 +376,86 @@ class Cons3rtClient:
         teams = json.loads(content)
         return teams
 
+    def list_project_members(self, project_id, max_results=40, page_num=0, state=None, role=None, username=None):
+        """Queries CONS3RT for a list of members for a project
+
+        :param project_id: (int) ID of the project
+        :param max_results: (int) maximum results to provide in the response
+        :param page_num: (int) page number to return
+        :param state: (str) membership state "REQUESTED" "ACTIVE" "BLOCKED" "DELETED"
+        :param role: (str) membership role "ADMINISTRATOR" "ASSET_RESTORER" "STATUS_READER" "UI_MACHINE" "TEST_TOOL"
+            "MEMBER" "CONSUMER" "STANDARD" "SOFTWARE_DEVELOPER" "TEST_DEVELOPER" "ASSET_SHARER" "ASSET_PROMOTER"
+            "POWER_SCHEDULE_UPDATER" "PROJECT_OWNER" "PROJECT_MANAGER" "PROJECT_MODERATOR" "REMOTE_ACCESS"
+            "MAESTRO_MACHINE" "FAP_MACHINE" "SCHEDULER_MACHINE" "CONS3RT_MACHINE" "SOURCEBUILDER_MACHINE"
+            "SYSTEM_ASSET_IMPORTER" "ASSET_CERTIFIER" "ASSET_UPLOADER"
+        :param username: (str) username to search for
+        :return: (list) of projects
+        :raises: Cons3rtClientError
+        """
+        target = 'projects/{i}/members?maxresults={m}&page={p}'.format(
+            i=str(project_id),
+            m=str(max_results),
+            p=str(page_num)
+        )
+        if state:
+            target += '&membershipState={s}'.format(s=state)
+        if role:
+            target += '&role={r}'.format(r=role)
+        if username:
+            target += '&name={n}'.format(n=username)
+        response = self.http_client.http_get(rest_user=self.user, target=target)
+        content = self.http_client.parse_response(response=response)
+        members = json.loads(content)
+        return members
+
+    def list_all_project_members(self, project_id, state=None, role=None, username=None):
+        """List all project members matching the provided search parameters
+
+        :param project_id: (int) ID of the project
+        :param state: (str) membership state "REQUESTED" "ACTIVE" "BLOCKED" "DELETED"
+        :param role: (str) membership role "ADMINISTRATOR" "ASSET_RESTORER" "STATUS_READER" "UI_MACHINE" "TEST_TOOL"
+            "MEMBER" "CONSUMER" "STANDARD" "SOFTWARE_DEVELOPER" "TEST_DEVELOPER" "ASSET_SHARER" "ASSET_PROMOTER"
+            "POWER_SCHEDULE_UPDATER" "PROJECT_OWNER" "PROJECT_MANAGER" "PROJECT_MODERATOR" "REMOTE_ACCESS"
+            "MAESTRO_MACHINE" "FAP_MACHINE" "SCHEDULER_MACHINE" "CONS3RT_MACHINE" "SOURCEBUILDER_MACHINE"
+            "SYSTEM_ASSET_IMPORTER" "ASSET_CERTIFIER" "ASSET_UPLOADER"
+        :param username: (str) username to search for
+        :return: (list) of projects
+        :raises: Cons3rtClientError
+        """
+        members = []
+        page_num = 0
+        max_results = 40
+        info_msg = 'project [{i}]'.format(i=str(project_id))
+        if state:
+            info_msg += ', membership state [{s}]'.format(s=state)
+        if role:
+            info_msg += ', project role [{r}]'.format(r=role)
+        if username:
+            info_msg += ', username [{n}]'.format(n=username)
+        while True:
+            msg = 'Retrieving members page [{p}]: '.format(p=str(page_num)) + info_msg
+            print(msg)
+            try:
+                page_of_members = self.list_project_members(
+                    project_id=project_id,
+                    max_results=max_results,
+                    page_num=page_num,
+                    state=state,
+                    role=role,
+                    username=username
+                )
+            except Cons3rtClientError as exc:
+                msg = 'Problem getting members page [{p}], max results [{m}] from: {i}'.format(
+                    p=str(page_num), m=str(max_results), i=info_msg)
+                raise Cons3rtClientError(msg) from exc
+            members += page_of_members
+            if len(page_of_members) < max_results:
+                break
+            else:
+                page_num += 1
+            print('Found {n} members in: {i}'.format(n=str(len(members)), i=info_msg))
+        return members
+
     def list_expanded_projects(self, max_results=40, page_num=0):
         """Queries CONS3RT for a list of projects the user is not a member of
 
@@ -478,16 +558,96 @@ class Cons3rtClient:
         system_details = json.loads(content)
         return system_details
 
-    def list_scenarios(self):
-        """Queries CONS3RT for a list of all scenarios
+    def list_system_designs(self, max_results=40, page_num=0):
+        """Returns a list of systems
 
+        :param max_results: (int) maximum number of results to retrieve
+        :param page_num: (int) page number to return
         :return: (list) Containing Scenario info
         """
-        response = self.http_client.http_get(rest_user=self.user, target='scenarios?maxresults=0')
+        response = self.http_client.http_get(
+            rest_user=self.user,
+            target='systems?maxresults={m}&page={p}'.format(
+                m=str(max_results),
+                p=str(page_num)
+            ))
+        content = self.http_client.parse_response(response=response)
+        systems = json.loads(content)
+        return systems
+
+    def list_all_system_designs(self):
+        """Returns a list of all system designs
+
+        :return: (list) of system designs
+        :raises: Cons3rtClientError
+        """
+        system_designs = []
+        page_num = 0
+        max_results = 40
+        while True:
+            print('Retrieving system designs: page {p}'.format(p=str(page_num)))
+            try:
+                page_of_system_designs = self.list_system_designs(
+                    max_results=max_results,
+                    page_num=page_num
+                )
+            except Cons3rtClientError as exc:
+                msg = 'Problem querying CONS3RT for a list of system designs, ' \
+                      'page: {p}, max results: {m}'.format(p=str(page_num), m=str(max_results))
+                raise Cons3rtClientError(msg) from exc
+            system_designs += page_of_system_designs
+            if len(page_of_system_designs) < max_results:
+                break
+            else:
+                page_num += 1
+            print('Found {n} system designs'.format(n=str(len(system_designs))))
+        return system_designs
+
+    def list_scenarios(self, max_results=40, page_num=0):
+        """Queries CONS3RT for a list of all scenarios
+
+        :param max_results: (int) maximum number of results to retrieve
+        :param page_num: (int) page number to return
+        :return: (list) Containing Scenario info
+        """
+        response = self.http_client.http_get(
+            rest_user=self.user,
+            target='scenarios?maxresults={m}&page={p}'.format(
+                m=str(max_results),
+                p=str(page_num)
+            ))
         content = self.http_client.parse_response(response=response)
         scenarios = json.loads(content)
         return scenarios
 
+    def list_all_scenarios(self):
+        """Returns a list of all scenarios
+
+        :return: (list) of scenarios
+        :raises: Cons3rtClientError
+        """
+        scenarios = []
+        page_num = 0
+        max_results = 40
+        while True:
+            print('Retrieving scenarios: page {p}'.format(p=str(page_num)))
+            try:
+                page_of_scenarios = self.list_system_designs(
+                    max_results=max_results,
+                    page_num=page_num
+                )
+            except Cons3rtClientError as exc:
+                msg = 'Problem querying CONS3RT for a list of scenarios, ' \
+                      'page: {p}, max results: {m}'.format(p=str(page_num), m=str(max_results))
+                raise Cons3rtClientError(msg) from exc
+            scenarios += page_of_scenarios
+            if len(page_of_scenarios) < max_results:
+                break
+            else:
+                page_num += 1
+            print('Found {n} scenarios'.format(n=str(len(scenarios))))
+        return scenarios
+
     def get_scenario_details(self, scenario_id):
         """Queries CONS3RT for details of a scenario ID
 
@@ -572,23 +732,32 @@ class Cons3rtClient:
                 result = vr['id']
         return result
 
-    def list_virtualization_realms_for_cloud(self, cloud_id, max_results=40, page_num=0):
-        """Queries CONS3RT for a list of Virtualization Realms for a specified Cloud ID
+    def list_all_virtualization_realms(self):
+        """Returns a list of virtualization realms
 
-        :param cloud_id: (int) Cloud ID to query
-        :param max_results: (int) maximum number of results to retrieve
-        :param page_num: (int) page number to return
         :return: (list) of virtualization realms
+        :raises: Cons3rtClientError
         """
-        response = self.http_client.http_get(
-            rest_user=self.user,
-            target='clouds/{i}/virtualizationrealms?maxresults={m}&page={p}'.format(
-                i=str(cloud_id),
-                m=str(max_results),
-                p=str(page_num)
-            ))
-        content = self.http_client.parse_response(response=response)
-        vrs = json.loads(content)
+        vrs = []
+        page_num = 0
+        max_results = 40
+        while True:
+            print('Retrieving virtualization realms: page {p}'.format(p=str(page_num)))
+            try:
+                page_of_vrs = self.list_virtualization_realms(
+                    max_results=max_results,
+                    page_num=page_num
+                )
+            except Cons3rtClientError as exc:
+                msg = 'Problem querying CONS3RT for a list of virtualization realms, ' \
+                      'page: {p}, max results: {m}'.format(p=str(page_num), m=str(max_results))
+                raise Cons3rtClientError(msg) from exc
+            vrs += page_of_vrs
+            if len(page_of_vrs) < max_results:
+                break
+            else:
+                page_num += 1
+            print('Found {n} virtualization realms'.format(n=str(len(vrs))))
         return vrs
 
     def list_all_virtualization_realms_for_cloud(self, cloud_id):
@@ -622,25 +791,6 @@ class Cons3rtClient:
             print('Found {n} virtualization realms in cloud {i}...'.format(n=str(len(vrs)), i=str(cloud_id)))
         return vrs
 
-    def list_virtualization_realms_for_project(self, project_id, max_results=40, page_num=0):
-        """Queries CONS3RT for a list of Virtualization Realms for a specified project ID
-
-        :param project_id: (int) project ID to query
-        :param max_results: (int) maximum number of results to retrieve
-        :param page_num: (int) page number to return
-        :return: (list) of virtualization realms
-        """
-        response = self.http_client.http_get(
-            rest_user=self.user,
-            target='projects/{i}/virtualizationrealms?maxresults={m}&page={p}'.format(
-                i=str(project_id),
-                m=str(max_results),
-                p=str(page_num)
-            ))
-        content = self.http_client.parse_response(response=response)
-        vrs = json.loads(content)
-        return vrs
-
     def list_all_virtualization_realms_for_project(self, project_id):
         """Returns a list of virtualization realms in the provided project ID
 
@@ -672,25 +822,6 @@ class Cons3rtClient:
             print('Found {n} virtualization realms in project {i}...'.format(n=str(len(vrs)), i=str(project_id)))
         return vrs
 
-    def list_virtualization_realms_for_team(self, team_id, max_results=40, page_num=0):
-        """Queries CONS3RT for a list of Virtualization Realms for a specified team ID
-
-        :param team_id: (int) project ID to query
-        :param max_results: (int) maximum number of results to retrieve
-        :param page_num: (int) page number to return
-        :return: (list) of virtualization realms
-        """
-        response = self.http_client.http_get(
-            rest_user=self.user,
-            target='teams/{i}/virtualizationrealms?maxresults={m}&page={p}'.format(
-                i=str(team_id),
-                m=str(max_results),
-                p=str(page_num)
-            ))
-        content = self.http_client.parse_response(response=response)
-        vrs = json.loads(content)
-        return vrs
-
     def list_all_virtualization_realms_for_team(self, team_id):
         """Returns a list of virtualization realms in the provided team ID
 
@@ -722,6 +853,80 @@ class Cons3rtClient:
             print('Found {n} virtualization realms in team {i}...'.format(n=str(len(vrs)), i=str(team_id)))
         return vrs
 
+    def list_virtualization_realms(self, max_results=40, page_num=0):
+        """Queries CONS3RT for a list of Virtualization Realms for a specified team ID
+
+        :param max_results: (int) maximum number of results to retrieve
+        :param page_num: (int) page number to return
+        :return: (list) of virtualization realms
+        """
+        response = self.http_client.http_get(
+            rest_user=self.user,
+            target='virtualizationrealms?maxresults={m}&page={p}'.format(
+                m=str(max_results),
+                p=str(page_num)
+            ))
+        content = self.http_client.parse_response(response=response)
+        vrs = json.loads(content)
+        return vrs
+
+    def list_virtualization_realms_for_cloud(self, cloud_id, max_results=40, page_num=0):
+        """Queries CONS3RT for a list of Virtualization Realms for a specified Cloud ID
+
+        :param cloud_id: (int) Cloud ID to query
+        :param max_results: (int) maximum number of results to retrieve
+        :param page_num: (int) page number to return
+        :return: (list) of virtualization realms
+        """
+        response = self.http_client.http_get(
+            rest_user=self.user,
+            target='clouds/{i}/virtualizationrealms?maxresults={m}&page={p}'.format(
+                i=str(cloud_id),
+                m=str(max_results),
+                p=str(page_num)
+            ))
+        content = self.http_client.parse_response(response=response)
+        vrs = json.loads(content)
+        return vrs
+
+    def list_virtualization_realms_for_project(self, project_id, max_results=40, page_num=0):
+        """Queries CONS3RT for a list of Virtualization Realms for a specified project ID
+
+        :param project_id: (int) project ID to query
+        :param max_results: (int) maximum number of results to retrieve
+        :param page_num: (int) page number to return
+        :return: (list) of virtualization realms
+        """
+        response = self.http_client.http_get(
+            rest_user=self.user,
+            target='projects/{i}/virtualizationrealms?maxresults={m}&page={p}'.format(
+                i=str(project_id),
+                m=str(max_results),
+                p=str(page_num)
+            ))
+        content = self.http_client.parse_response(response=response)
+        vrs = json.loads(content)
+        return vrs
+
+    def list_virtualization_realms_for_team(self, team_id, max_results=40, page_num=0):
+        """Queries CONS3RT for a list of Virtualization Realms for a specified team ID
+
+        :param team_id: (int) project ID to query
+        :param max_results: (int) maximum number of results to retrieve
+        :param page_num: (int) page number to return
+        :return: (list) of virtualization realms
+        """
+        response = self.http_client.http_get(
+            rest_user=self.user,
+            target='teams/{i}/virtualizationrealms?maxresults={m}&page={p}'.format(
+                i=str(team_id),
+                m=str(max_results),
+                p=str(page_num)
+            ))
+        content = self.http_client.parse_response(response=response)
+        vrs = json.loads(content)
+        return vrs
+
     def add_virtualization_realm_admin(self, vr_id, username):
         response = self.http_client.http_put(
             rest_user=self.user,
@@ -1281,10 +1486,14 @@ class Cons3rtClient:
         result = self.http_client.parse_response(response=response)
         return result
 
-    def delete_asset(self, asset_id):
+    def delete_asset(self, asset_id, force=False):
+        if force:
+            force_str = 'true'
+        else:
+            force_str = 'false'
         response = self.http_client.http_delete(
             rest_user=self.user,
-            target='assets/{i}'.format(i=str(asset_id))
+            target='assets/{i}?force={f}'.format(i=str(asset_id), f=force_str)
         )
         result = self.http_client.parse_response(response=response)
         return result
@@ -1490,6 +1699,52 @@ class Cons3rtClient:
             print('Found {n} users...'.format(n=str(len(users))))
         return users
 
+    def retrieve_container_asset(self, asset_id):
+        """Retrieves details for the container asset
+
+        :param asset_id: (int) asset ID
+        :return: (dict) details about the container asset
+        :raises: Cons3rtClientError
+        """
+        target = 'containers/{i}'.format(i=str(asset_id))
+        try:
+            response = self.http_client.http_get(
+                rest_user=self.user,
+                target=target
+            )
+        except Cons3rtClientError as exc:
+            raise Cons3rtClientError('Problem retrieving container asset: {i}'.format(i=str(asset_id))) from exc
+        try:
+            result = self.http_client.parse_response(response=response)
+        except Cons3rtClientError as exc:
+            msg = 'The HTTP response contains a bad status code'
+            raise Cons3rtClientError(msg) from exc
+        software_asset = json.loads(result)
+        return software_asset
+
+    def retrieve_software_asset(self, asset_id):
+        """Retrieves details for the software asset
+
+        :param asset_id: (int) asset ID
+        :return: (dict) details about the software asset
+        :raises: Cons3rtClientError
+        """
+        target = 'software/{i}'.format(i=str(asset_id))
+        try:
+            response = self.http_client.http_get(
+                rest_user=self.user,
+                target=target
+            )
+        except Cons3rtClientError as exc:
+            raise Cons3rtClientError('Problem retrieving software asset: {i}'.format(i=str(asset_id))) from exc
+        try:
+            result = self.http_client.parse_response(response=response)
+        except Cons3rtClientError as exc:
+            msg = 'The HTTP response contains a bad status code'
+            raise Cons3rtClientError(msg) from exc
+        software_asset = json.loads(result)
+        return software_asset
+
     def retrieve_software_assets(self, asset_type=None, community=False, category_ids=None, expanded=False,
                                  max_results=40, page_num=0):
         """Get a list of software assets
